#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QApplication>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi (this);
    // displayObject (ui->pushButton);
    displayTree (ui->treeWidget->invisibleRootItem(), this);
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::displayTree (QTreeWidgetItem * target, QObject * obj)
{
    auto node = new ObjectItem;
    const QMetaObject * type = obj->metaObject();
    QString s = obj->objectName() + " : " + type->className();
    node->setText (0, s);
    node->setObject (obj);
    node->setForeground(0, QColor ("blue"));
    target->addChild (node);
    displayDetail (node, obj);

    const QObjectList & list = obj->children();
    for (QObject * t : list)
        displayTree (node, t);
}

void MainWindow::displayDetail (QTreeWidgetItem * target, QObject * obj)
{
    const QMetaObject * type = obj->metaObject();

    QTreeWidgetItem * branch = new QTreeWidgetItem ();
    branch->setText(0, "methods");
    branch->setForeground(0, QColor ("lime"));
    target->addChild (branch);

    int cnt = type->methodCount();
    for (int i = 0; i < cnt; i++)
    {
       QMetaMethod meth = type->method(i);
       MethodItem * item = new MethodItem;
       item->setMethod (meth);
       item->setForeground(0, QColor ("lime"));
       item->setText(0, meth.name());
       QString t = QString (meth.typeName()) + " " + meth.methodSignature();
       item->setToolTip(0, t);
       branch->addChild (item);
    }
}

QTableWidgetItem * MainWindow::displayLine (QString name, QString value)
{
    int line = line_count;
    line_count ++;
    if (ui->tableWidget->rowCount() < line_count)
        ui->tableWidget->setRowCount (line_count);

    QTableWidgetItem * item = new QTableWidgetItem;
    item->setText (name);
    item->setForeground(QColor ("blue"));
    ui->tableWidget->setItem (line, 0, item);

    item = new QTableWidgetItem;
    item->setText (value);
    item->setForeground(QColor ("blue"));
    ui->tableWidget->setItem (line, 1, item);

    return item;
}

void MainWindow::displayProperty (const QObject * obj, QMetaProperty prop)
{
    QString name = prop.name();
    QVariant value = prop.read (obj);
    QTableWidgetItem * item = displayLine (name, value.toString());

    QVariant::Type t = value.type();
    if (t == QVariant::Bool)
    {
       bool b = value.toBool ();
       item->setData (Qt::CheckStateRole, b ? Qt::Checked : Qt::Unchecked);
    }
    if (t == QVariant::Color)
    {
        QColor c = value.value <QColor> ();
        item->setData (Qt::DecorationRole, c);
    }
    if (t == QVariant::Palette)
    {
        QPalette p = value.value <QPalette> ();
        QColor c = p.background().color();
        item->setData (Qt::DecorationRole, c);
    }
    if (t == QVariant::Icon)
    {
        QIcon i = value.value <QIcon> ();
        item->setData (Qt::DecorationRole, i);
    }
    if (t == QVariant::Image)
    {
        QImage i = value.value <QImage> ();
        item->setData (Qt::DecorationRole, i);
    }
    // if (t == QVariant::Color)
}

void MainWindow::displayObject (QObject * obj)
{
    line_count = 0;
    ui->tableWidget->setColumnCount(2);
    ui->tableWidget->setRowCount(10);

    QStringList list;
    list << "Name" << "Value";
    ui->tableWidget->setHorizontalHeaderLabels (list);

    const QMetaObject * type = obj->metaObject ();
    displayLine("className", type->className());

    int cnt = type->propertyCount();
    for (int inx = 0; inx < cnt; inx++)
    {
        // #include <QMetaProperty>
        QMetaProperty prop = type->property (inx);
        displayProperty (obj, prop);
    }
}

int main(int argc, char *argv[])
{
    QApplication a(argc, argv);
    MainWindow w;
    w.show();

    return a.exec();
}
